// hexmod.cpp: определяет точку входа для консольного приложения.
//

#include "stdafx.h"
#include "util.h"
#include "intelhexrecord.h"
#include "cmemorymodel.h"
#include <boost/program_options.hpp>

namespace po = boost::program_options;

int _tmain(int argc, _TCHAR* argv[])
{
    int app_res = 0;
    std::string ofile;
    std::string ifile;
    int addr;
    std::string val;
    try
    {
        po::options_description desc("Allowed options");
        desc.add_options()
        ("help,h", "print usage message")
        ("input,i", po::value(&ifile), "pathname for input")
        ("output,o", po::value(&ofile), "pathname for output")
        ("address,a", po::value(&addr), "address")
        ("value,v", po::value(&val), "value for write")
        ;

        po::variables_map vm;
        po::store(parse_command_line(argc, argv, desc), vm);

        if (vm.count("help"))
        {
            std::cout << desc << std::endl;
            return 0;
        }
        /*std::cout<<vm["input"].as<std::string>()<<std::endl;
        std::cout<<vm["output"].as<std::string>()<<std::endl;
        std::cout<<vm["address"].as<int>()<<std::endl;
        std::cout<<vm["value"].as<std::string>()<<std::endl;*/

        //открытие файла
        std::ifstream in_file(vm["input"].as<std::string>());
        if(!in_file)
        {
            throw std::exception("File not opened");
        }

        //модель памяти
        CMemoryModel memory_model;

        while(!in_file.eof())
        {
            std::string in_str;
            in_file>>in_str;
            //std::cout<<in_str<<std::endl;

            CAbstractIntelHexRecord * rec = CAbstractIntelHexRecord::createRecord(in_str);
            if(rec)
                {
                    memory_model.insert(rec->addr(),rec->data());
                    //std::cout<<rec->toString()<<std::endl;
                    //std::cout<<"added "<<rec->data().size()<<" bytes for addr "<<rec->addr()<<std::endl;
                    //std::cout<<"memoty model size "<<memory_model.size()<<std::endl;
                }

        }
        in_file.close();

        //std::cout<<"memoty model size "<<memory_model.size()<<std::endl;

        //std::cout<<"Hex test : "<<static_cast<int>(Util::convertHex("16"))<<" "<<static_cast<int>(Util::convertHex("Aa"))<<std::endl;
        //std::cout<<"Hex test : "<<Util::convertToHex(static_cast<unsigned char>(22))<<" "<<Util::convertToHex(0xaa)<<std::endl;


        std::vector<CAbstractIntelHexRecord*> records;
        //test memory
        /*std::vector<CAbstractIntelHexRecord*> records = memory_model.toRecords();
        for(int i = 0; i < records.size(); ++i)
            {
                std::cout<<records[i]->toString()<<std::endl;
            }*/

        std::vector<unsigned char> bin;
        if( (vm["value"].as<std::string>().size() == 0) || ((vm["value"].as<std::string>().size()%2) != 0))//строка должна иметь четное кол-во символов
            {
                std::cout<<"Wrong walue" <<std::endl;
                return 0;
            }

        for(unsigned int idx = 0; idx < (vm["value"].as<std::string>().size()-1) ; idx+=2 )
        {
            bin.push_back(Util::convertHex(vm["value"].as<std::string>().substr(idx,2)));
        }

        //замена значения
        memory_model.owerwrite(vm["address"].as<int>(),bin);
        records = memory_model.toRecords();

        /*records = memory_model.toRecords();
        for(int i = 0; i < records.size(); ++i)
            {
                std::cout<<records[i]->toString()<<std::endl;
            }*/

        std::ofstream out_file(vm["output"].as<std::string>(),std::ios_base::trunc | std::ios_base::out);
        if(!out_file)
        {
            throw std::exception("File not opened");
        }

        for(unsigned int i = 0; i < records.size(); ++i)
            {
                out_file<<records[i]->toString()<<std::endl;
            }


    }
    catch(std::exception& e)
    {
            std::cout<<"Error : "<<e.what()<<std::endl;
            app_res = 127;
    }
    catch(Util::ErrorMsg * err)
    {
        std::cout<<"Error : "<<err->What()<<std::endl;
        app_res = 126;
    }
    catch(...)
    {
            app_res = 128;
    }
    return app_res;
}

