
#include "stdafx.h"
#include "intelhexrecord.h"
#include "util.h"
#include <ostream>
#include <algorithm>

CAbstractIntelHexRecord* CAbstractIntelHexRecord::createRecord(EIntelHexRecordType record_type)
{
    CAbstractIntelHexRecord * res = 0;
    switch(record_type)
    {
    case HRT_BINATY:   res = new CIntelBinRecord(); break;
    case HRT_END_FILE: res = new CIntelEndFileRecord(); break;
    }
    res->setType(record_type);
    return res;
}

CAbstractIntelHexRecord* CAbstractIntelHexRecord::createRecord(const std::string& str)
{
    CAbstractIntelHexRecord* res = 0;

    std::vector<unsigned char> bin;
    //1 - разделитель строки - двоеточие
    if( (str.size() == 0) || ((str.size()%2) == 0))//строка должна иметь нечетное кол-во символов
    {
        return 0;
    }
    if(str[0] != ':')
    {
        return 0;
    }

    for(unsigned int idx = 1; idx < (str.size()-1) ; idx+=2 )
    {
        bin.push_back(Util::convertHex(str.substr(idx,2)));
    }

    if(bin.size()-5 != bin[0])
    {
        std::cout<<"error len: "<<"size - >"<<bin.size()<<" "<<static_cast<int>(bin[0])<<std::endl;
        return 0;
    }

    //подсчет контрольной сумы
    unsigned char checksum = bin[0];
    for(unsigned int idx = 1; idx < bin.size()-1; ++idx)
    {
        checksum += bin[idx];
    }
    checksum = 0 - checksum; // Final checksum preparation
    if( checksum != bin[bin.size()-1] )
    {
        std::cout<<"error checksum. "<<Util::convertToHex(checksum)<<std::endl;
        return 0;
    }

    bin.pop_back();//удаляем контрольную сумму
    res = createRecord(static_cast<EIntelHexRecordType>(bin[3]));
    if(res)
    {
        res->setRawData(bin);
    }

    return res;
}

EIntelHexRecordType CAbstractIntelHexRecord::type()
{
    if(data_.size() < 4) return HRT_INVALID_RECORD;

    return static_cast<EIntelHexRecordType>(data_.at(3));
}

void CAbstractIntelHexRecord::setType(EIntelHexRecordType t)
{
    data_[3] = static_cast<unsigned char>(t);
}

unsigned int CAbstractIntelHexRecord::addr()
{
    return ( (data_[1]<<8)+data_[2] );
}

unsigned int CAbstractIntelHexRecord::record_size()
{
    return data_[0];
}

void CAbstractIntelHexRecord::setAddr(int a)
{
    data_[1] = (unsigned char) ((a >> 8) & 0xff);
    data_[2] = (unsigned char) (a & 0xff);
}

void CAbstractIntelHexRecord::setData(std::vector<unsigned char> &data)
{
    data_[0] = data.size();
    data_.resize(4+data.size());
    std::copy(data.begin(),data.end(),data_.begin()+4);
}

std::vector<unsigned char> CAbstractIntelHexRecord::data()
{
    std::vector<unsigned char> res;
    res.resize(record_size());
    std::copy(data_.begin()+4,data_.end(),res.begin());
    return res;
}

void CAbstractIntelHexRecord::setRawData(std::vector<unsigned char> &data)
{
    data_ = data;
}

CIntelBinRecord::CIntelBinRecord() : CAbstractIntelHexRecord()
{
}


std::string CIntelBinRecord::toString()
{
    std::string res;
    res.push_back(':');
    unsigned char checksum = 0;
    std::for_each(data_.begin(),data_.end(),
                  [&,this](unsigned char b)
                    {res.append(Util::convertToHex(b));
                     checksum+=b;
                    });
    checksum = 0 - checksum;
    res+=Util::convertToHex(checksum);
    return res;
}


CIntelEndFileRecord::CIntelEndFileRecord() : CAbstractIntelHexRecord()
{
}

std::string CIntelEndFileRecord::toString()
{
    return std::string(":00000001FF");
}


